@@ -22,7 +22,6 @@
 #include "nsMsgUtils.h"
 #include "nsIImapIncomingServer.h"
 #include "nsIMsgFilterPlugin.h"
-#include "nsIStringBundle.h"
 #include "nsCOMArray.h"
 #include "nsTArray.h"
 #include "nsTHashtable.h"
@@ -31,8 +30,6 @@
 #include "nsIWeakReferenceUtils.h"
 #include "nsMsgEnumerator.h"
 
-#define MESSENGER_STRING_URL "chrome://messenger/locale/messenger.properties"
-
 typedef AutoTArray<nsMsgViewIndex, 1> nsMsgViewIndexArray;
 static_assert(nsMsgViewIndex(nsMsgViewIndexArray::NoIndex) ==
                   nsMsgViewIndex_None,
@@ -118,25 +115,36 @@ class nsMsgDBView : public nsIMsgDBView,
  protected:
   virtual ~nsMsgDBView();
 
-  static nsString kHighestPriorityString;
-  static nsString kHighPriorityString;
-  static nsString kLowestPriorityString;
-  static nsString kLowPriorityString;
-  static nsString kNormalPriorityString;
+  static mozilla::StaticAutoPtr<nsString> sHighestPriorityString;
+  static mozilla::StaticAutoPtr<nsString> sHighPriorityString;
+  static mozilla::StaticAutoPtr<nsString> sLowestPriorityString;
+  static mozilla::StaticAutoPtr<nsString> sLowPriorityString;
+  static mozilla::StaticAutoPtr<nsString> sNormalPriorityString;
 
-  static nsString kReadString;
-  static nsString kRepliedString;
-  static nsString kForwardedString;
-  static nsString kRedirectedString;
-  static nsString kNewString;
+  static mozilla::StaticAutoPtr<nsString> sReadString;
+  static mozilla::StaticAutoPtr<nsString> sRepliedString;
+  static mozilla::StaticAutoPtr<nsString> sForwardedString;
+  static mozilla::StaticAutoPtr<nsString> sRedirectedString;
+  static mozilla::StaticAutoPtr<nsString> sNewString;
 
   // Used for group views.
-  static nsString kTodayString;
-  static nsString kYesterdayString;
-  static nsString kLastWeekString;
-  static nsString kTwoWeeksAgoString;
-  static nsString kOldMailString;
-  static nsString kFutureDateString;
+  static mozilla::StaticAutoPtr<nsString> sTodayString;
+  static mozilla::StaticAutoPtr<nsString> sYesterdayString;
+  static mozilla::StaticAutoPtr<nsString> sLastWeekString;
+  static mozilla::StaticAutoPtr<nsString> sTwoWeeksAgoString;
+  static mozilla::StaticAutoPtr<nsString> sOldMailString;
+  static mozilla::StaticAutoPtr<nsString> sFutureDateString;
+
+  // Also used for group views.
+  static mozilla::StaticAutoPtr<nsString> sNoStatusString;
+  static mozilla::StaticAutoPtr<nsString> sUntaggedString;
+  static mozilla::StaticAutoPtr<nsString> sNoPriorityString;
+  static mozilla::StaticAutoPtr<nsString> sNoAttachmentsString;
+  static mozilla::StaticAutoPtr<nsString> sAttachmentsString;
+  static mozilla::StaticAutoPtr<nsString> sNotStarredString;
+  static mozilla::StaticAutoPtr<nsString> sStarredString;
+
+  static mozilla::StaticAutoPtr<nsString> sAndOthersString;
 
   RefPtr<mozilla::dom::XULTreeElement> mTree;
   nsCOMPtr<nsIMsgJSTree> mJSTree;
@@ -144,7 +152,6 @@ class nsMsgDBView : public nsIMsgDBView,
   // Set when we're telling the outline a row is being removed. Used to
   // suppress msg loading during delete/move operations.
   bool mRemovingRow;
-  bool mCommandsNeedDisablingBecauseOfSelection;
   bool mSuppressChangeNotification;
 
   nsresult FetchAuthor(nsIMsgDBHdr* aHdr, nsAString& aAuthorString);
@@ -388,7 +395,8 @@ class nsMsgDBView : public nsIMsgDBView,
                                bool* resultToggleState);
   bool OfflineMsgSelected(nsTArray<nsMsgViewIndex> const& selection);
   bool NonDummyMsgSelected(nsTArray<nsMsgViewIndex> const& selection);
-  static void GetString(const char16_t* aStringName, nsAString& aValue);
+  static void GetString(mozilla::intl::Localization* l10n, nsACString const& id,
+                        nsAString& value);
   static nsresult GetPrefLocalizedString(const char* aPrefName,
                                          nsString& aResult);
   nsresult AppendKeywordProperties(const nsACString& keywords,
@@ -461,7 +469,6 @@ class nsMsgDBView : public nsIMsgDBView,
   nsWeakPtr mMsgWindowWeak;
   // We push command update notifications to the UI from this.
   nsWeakPtr mCommandUpdater;
-  static nsCOMPtr<nsIStringBundle> mMessengerStringBundle;
 
   // Used to determine when to start and end junk plugin batches.
   uint32_t mNumMessagesRemainingInBatch;
@@ -503,7 +510,7 @@ class nsMsgDBView : public nsIMsgDBView,
   static nsDateFormatSelectorComm m_dateFormatDefault;
   static nsDateFormatSelectorComm m_dateFormatThisWeek;
   static nsDateFormatSelectorComm m_dateFormatToday;
-  static nsString m_connectorPattern;
+  static mozilla::StaticAutoPtr<nsString> sConnectorPattern;
 
   bool ServerSupportsFilterAfterTheFact();
 
