@@ -44,9 +44,7 @@
 #include "mozilla/Services.h"
 #include "mozilla/StaticPtr.h"
 #include "mozilla/TimeStamp.h"
-#include "mozilla/TypedEnumBits.h"
 #include "mozilla/UniquePtr.h"
-#include "mozilla/Unused.h"
 #include "mozilla/dom/AutoEntryScript.h"
 #include "mozilla/dom/BindingDeclarations.h"
 #include "mozilla/dom/CallbackObject.h"
@@ -1214,7 +1212,7 @@ void nsMessageManagerScriptExecutor::LoadScriptInternal(
         if (!envChain.append(aMessageManager)) {
           return;
         }
-        Unused << JS_ExecuteScript(cx, envChain, script, &rval);
+        (void)JS_ExecuteScript(cx, envChain, script, &rval);
       }
     }
   }
