@@ -53,10 +53,6 @@ const WebIDLPrefFunc sWebIDLPrefs[] = {
   StaticPrefs::dom_enable_web_task_scheduling,
   // dom.event.pointer.rawupdate.enabled
   StaticPrefs::dom_event_pointer_rawupdate_enabled,
-  // dom.events.asyncClipboard.clipboardItem
-  StaticPrefs::dom_events_asyncClipboard_clipboardItem,
-  // dom.events.script_execute.enabled
-  StaticPrefs::dom_events_script_execute_enabled,
   // dom.fetchKeepalive.enabled
   StaticPrefs::dom_fetchKeepalive_enabled,
   // dom.forms.autocorrect
@@ -133,8 +129,6 @@ const WebIDLPrefFunc sWebIDLPrefs[] = {
   StaticPrefs::dom_textMetrics_emHeight_enabled,
   // dom.textMetrics.fontBoundingBox.enabled
   StaticPrefs::dom_textMetrics_fontBoundingBox_enabled,
-  // dom.text_fragments.create_text_fragment.enabled
-  StaticPrefs::dom_text_fragments_create_text_fragment_enabled,
   // dom.text_fragments.enabled
   StaticPrefs::dom_text_fragments_enabled,
   // dom.vibrator.enabled
@@ -173,6 +167,8 @@ const WebIDLPrefFunc sWebIDLPrefs[] = {
   StaticPrefs::layout_accessiblecaret_magnifier_enabled,
   // layout.css.anchor-positioning.enabled
   StaticPrefs::layout_css_anchor_positioning_enabled,
+  // layout.css.anchor-positioning.position-try-order.enabled
+  StaticPrefs::layout_css_anchor_positioning_position_try_order_enabled,
   // layout.css.convertFromNode.enabled
   StaticPrefs::layout_css_convertFromNode_enabled,
   // layout.css.field-sizing.enabled
@@ -213,8 +209,8 @@ const WebIDLPrefFunc sWebIDLPrefs[] = {
   StaticPrefs::layout_css_scroll_driven_animations_enabled,
   // layout.css.text-autospace.enabled
   StaticPrefs::layout_css_text_autospace_enabled,
-  // layout.css.text-decoration-trim.enabled
-  StaticPrefs::layout_css_text_decoration_trim_enabled,
+  // layout.css.text-decoration-inset.enabled
+  StaticPrefs::layout_css_text_decoration_inset_enabled,
   // layout.css.typed-om.enabled
   StaticPrefs::layout_css_typed_om_enabled,
   // layout.css.zoom.enabled
